/*
Mapping a Wii controller to send keystrokes

Utilizes the proCONTROLL library:
http://creativecomputing.cc/p5libs/procontroll/index.htm

Based on Dan Plows's sketch found in this blog post:
http://danplows.com/blog/map-ps3-controller-to-mac-via-bluetooh/

*/

import procontroll.*;
import guicomponents.*;
import java.awt.AWTException;

//Libraries for controlling mouse and keyboard
import java.awt.Robot;
import java.io.*;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import java.awt.MouseInfo;
Wiimote[] wiimotes = new Wiimote[0];

String fileName = "settings.txt";
String settings[] = new String[0];

boolean pMousePressed = false;

PFont font;

void setup(){
    size(400,300);
  //Setup GUI
    guiSetup();
  //Attempt to create an instacne of the "Robot" class
    try{
      robot = new Robot();
    }
    catch (AWTException e) {
      e.printStackTrace();
    }
    controll = ControllIO.getInstance(this);
  //Look for connected wiimotes
  for(int i=0; i<controll.getNumberOfDevices(); i++){
    ControllDevice tempDevice = controll.getDevice(i);
    if(tempDevice.getName().length() > 7){
      if(tempDevice.getName().substring(0, 7).equals("Wiimote")){
        wiimotes =  (Wiimote[])append(wiimotes, new Wiimote(tempDevice.getName()));
        //println(tempDevice.getName());
      }
    }
  }
  //Load Settings
  try{
    getSettings(fileName);
  }
  catch(Exception e){
    //Overwrite settings if loging fails
    saveSettings(fileName);
  }
  //Update status
  if(wiimotes.length > 0){
    statusBar.setText("Found " + wiimotes.length + " connected Wiimote(s)");
  }
  else{
    statusBar.setText("No Wiimotes are connected");
  }
  //Setup fonts
  font = createFont("Monaco", 14);
  textFont(font);
}

void draw(){
  background(#FFFFFF);
    //FiM Macro tests
      //fimMacros(0);
  //Loop for each wiimote
    for(int i=0; i<wiimotes.length; i++){
      //Draw
        stroke(#000000);
        if(wiimotes[i].setKey){
            fill(#00FFFF, 0x80);
          }
          else{
            noFill();
          }
        rect(i*70+10, 10, 65, 120);
        wiimotes[i].drawWiimote(i*70+45, 10);
        wiimotes[i].drawNunchuk(i*70+12, 30);
      wiimotes[i].loop();
      fill(#000000);
      text("#" + str(i+1), i*70+15, 25);
      //Toggle key assignment
        if(!mousePressed && pMousePressed && mouseButton == LEFT && i*70+10 < mouseX && mouseX < i*70+70 && 10 < mouseY && mouseY < 120){
          wiimotes[i].setKey = !wiimotes[i].setKey;
          if(wiimotes[i].setKey){
            statusBar.setText("Ready to set a key for Wiimote#" + str(i+1));
            fill(#FFFFFF);
          }
          else{
            statusBar.setText("Canceled key assignment for Wiimote#" + str(i+1));
            fill(#0000FF);
          }
        }
      //Button key bindings
        for(int j=0; j<28; j++){
          if(wiimotes[i].pressed(j)){
            keySent = false;
            pKeyCode = -1;
          }
          if(wiimotes[i].setKey){
            if(keySent){
              if(wiimotes[i].buttons[j].pressed() && pKeyCode > -1){
                wiimotes[i].buttonKeyBind(j, pKeyCode);
                statusBar.setText("Set button #" + j + " of Wiimote#" + i + " to keyCode " + keyCode);
                saveSettings(fileName);
                wiimotes[i].setKey = false;
                pKeyCode = -1;
              }
            }
            else{
              if(wiimotes[i].released(j)){
                wiimotes[i].buttonKeyBind(j, -1);
                statusBar.setText("Cleared button " + j + " of Wiimote#" + i);
                saveSettings(fileName);
                wiimotes[i].setKey = false;
              }
            }
          }
        }
    }
  pMousePressed = mousePressed;
}

void delay(){
  try{
    Thread.sleep(100);
  }
  catch(InterruptedException ie){ }
}
